/******************************************************************************\
*                                                                              *
* Network.DLL v1.54a                                                   Sandman *
*                                                                              *
* Should one want to edit and rerelease this source, a compiled version or a   *
* program using it, please be so kind as to inform me of your improvements or  *
* changes, so I'd be able to incorporate them into later versions. Should you  *
* use elements of this code, please be so kind as to credit this code as such. *
*                                                                              *
\******************************************************************************/

/* Includes */

	#include "fxdll.h"
	#include "stdlib.h"
	#include "SDL.h"
	#include "SDL_net.h"
	#include "math.h"
	#include "string.h"

/* Defines */

	// Maxima
	#define NET_MAX_SOCKETSETS					   1 // There's only one
	#define NET_MAX_MSG_LEN					   65536

	// Status codes
	#define NET_STATUS_DISCONNECTING		  -4 // Bridge is being destroyed
	#define NET_STATUS_DISCONNECTED			  -2 // Bridge was destroyed (No bridge)
	#define NET_STATUS_INACTIVE				   0 // No bridge
	#define NET_STATUS_ACTIVE				   1 // Incoming Message (Incoming only)
	#define NET_STATUS_ESTABLISHED			   2 // Bridge is enabled
	#define NET_STATUS_ESTABLISHING			   4 // Bridge is being enabled

	// Setup codes
	#define NET_CONSOLEREPORTS_NO			   0
	#define NET_CONSOLEREPORTS_YES			   1

	// Error codes
	#define NET_ERROR_NONE					   0
	#define NET_ERROR_INITIALIZATION		 -10
	#define NET_ERROR_NOTINITIALIZED		 -11
	#define NET_ERROR_INVALIDADDRESS		 -12
	#define NET_ERROR_RESOLVINGHOST			 -13
	#define NET_ERROR_ALLOCSOCKETSET		 -14
	#define NET_ERROR_INVALIDSOCKETSET		 -15
	#define NET_ERROR_SOCKETSETINACTIVE		 -16
	#define NET_ERROR_ADDINGSOCKET			 -17
	#define NET_ERROR_INVALIDSOCKET			 -18
	#define NET_ERROR_DELETINGSOCKET		 -19
	#define NET_ERROR_CONNECTING			 -20
	#define NET_ERROR_INVALIDCONN			 -21
	#define NET_ERROR_CONNINACTIVE			 -22
	#define NET_ERROR_LISTENINGONCONN		 -23
	#define NET_ERROR_TOOMANYCONNS			 -24
	#define NET_ERROR_TOOFEWCONNS			 -25
	#define NET_ERROR_INVALIDTYPE			 -26
	#define NET_ERROR_ALREADYINIT			 -27
	#define NET_ERROR_MESSAGETOOLONG		 -28
	#define NET_ERROR_MESSAGETOOSHORT		 -29
	#define NET_ERROR_OUTGOINGSTACKFULL		 -30
	#define NET_ERROR_INCOMINGSTACKFULL		 -31
	#define NET_ERROR_SENDING				 -32
	#define NET_ERROR_RECEIVING				 -33
	#define NET_ERROR_DISCONNECTING			 -34
	#define NET_ERROR_DISCONNECTED			 -35
	#define NET_ERROR_TOOMANYLISTENPORTS	 -36
	#define NET_ERROR_INVALIDPORTNUMBER		 -37
	#define NET_ERROR_SIZEMISMATCH			 -38
	#define NET_ERROR_RESOLVINGIP			 -39
	#define NET_ERROR_NONEXTERNALCONN		 -40
	#define NET_ERROR_NONEWMESSAGE			 -41
	#define NET_ERROR_UNFINISHEDMESSAGE		 -42
	#define NET_ERROR_NULLPOINTER			 -43

	// Error codes (Fenix Include generated only)
	#define NET_ERROR_NONEXISTINGGRAPH		 -80
	#define NET_ERROR_NONEXISTINGFILE		 -81
	#define NET_ERROR_CREATINGGRAPH			 -82
	#define NET_ERROR_OPENINGFILE			 -83

/* Functions */

	int NET_SocketSetAlloc	(Uint8 socketset, Uint16 size);
	int NET_SocketSetFree	(Uint8 socketset);
	int NET_Send			(Uint16 conn,Uint8* s, Uint32 len, Uint8 includeseparator);
	int NET_GetIncomingMsg	(Uint16 conn);
	int NET_Recv			(Uint16 conn, Uint8* s, Uint8 includeseparator);
	int NET_RecvVar			(Uint16 conn, Uint8* s, Uint32 expectedlen, Uint8 includeseparator);
	int NET_Listen			(Uint8 set,Uint16 port, Sint8 consolereports);
	int NET_Connect			(Uint8 set, Uint8* hostname, Uint16 port, Sint8 consolereports);
	int NET_Disconnect		(Uint16 conn);
	int NET_Accept			(Uint8 set, Uint16 inconn, TCPsocket socket);
	int NET_Separator		(Uint16 conn, Uint8* s, Uint8 len);

/* Typedefs */

	// Connection
	typedef struct{
		IPaddress	address;
		TCPsocket	socket;
		Sint16		socketset_id;
		Uint32		IDcounter;
		Sint8		consolereports;
		Uint8*		incomingmsg;
		Uint32		incomingmsglength;
		Uint8*		separator;
		Uint16		separatorlength;
//		Uint8		separatorincluded;
	}_con;

	// Socket Set
	typedef struct{
		Sint32				status;
		SDLNet_SocketSet	SocketSet;
		Uint16				size;
	}_socketset;

	// NET
	struct
	{
		Uint8		status;
		Sint32		timeout;
		TCPsocket	tempsocket;

		// Socket Set
		_socketset	SocketSet[NET_MAX_SOCKETSETS];

		// Connections 
		Uint16*		ExtConn; // external connidentifiers, only used in Fenix DLL commands (not yet used)
		_con*		Conn;
		Sint32*		Incoming;
		Sint32*		Status;

		// Settings
		Uint16		maxconnections; // == default socket size
		Uint16		maxlistenports;
		Uint16		sendportbegin;
		Uint16		sendportend;
		Uint8*		consolereports;
		Uint8*		returnnetdllcommands;

		// Status variables
		Uint16*		activeconnections;
		Uint16*		activelistenports;

		// Download/Upload Meter
		Sint32*		totaltransferrate_up;
		Sint32*		totaltransferrate_down;
		Sint32*		totaltransferred_up;
		Sint32*		totaltransferred_down;
		Sint32*		transferrate_up;   // for each connection
		Sint32*		transferrate_down; //
		Sint32*		transferred_up;    //
		Sint32*		transferred_down;  //

	} NET;

/* Constants */

	const int		NETversion			= 154;
	const Uint8*	NETversion_string	= "v1.54a\0";
	const Uint8*	NETversionreply		= "v1.54a, built on d23-m04-y07\0";

/********************************************************************\
*                                                                    *
* C NET Functions                                            Sandman *
*                                                                    *
\********************************************************************/

/**
 * NET_PrepareConn( connection_id )
 * Prepare connection conn.
 * Returns NET_ERROR_NONE.
 */
static int net_prepareconn (Uint16 conn)
{
	NET.Status[conn]	= NET_STATUS_INACTIVE;
	NET.Incoming[conn]	= NET_STATUS_INACTIVE;

	NET.transferrate_up  [conn] = 0;
	NET.transferrate_down[conn] = 0;
	NET.transferred_up   [conn] = 0;
	NET.transferred_down [conn] = 0;

	NET.Conn[conn].IDcounter		= 0;

	NET.Conn[conn].consolereports	= 0;

	NET.Conn[conn].incomingmsg		= malloc(1);
	NET.Conn[conn].incomingmsg[0] = '\0';
	NET.Conn[conn].incomingmsglength = 0;
	NET.Conn[conn].separator = malloc(1);
	NET.Conn[conn].separator[0] = '\0';
	NET.Conn[conn].separatorlength = 0;
	
	return NET_ERROR_NONE;	
}

/**
 * NET_ReleaseConn( connection_id )
 * Unprepare connection conn.
 * Returns NET_ERROR_NONE.
 */
int net_releaseconn (Uint16 conn)
{
	if(NET.Conn[conn].incomingmsg!=NULL)
		free(NET.Conn[conn].incomingmsg);
	NET.Conn[conn].incomingmsglength = 0;
	if(NET.Conn[conn].separator!=NULL)
		free(NET.Conn[conn].separator);
	NET.Conn[conn].separatorlength = 0;
	return NET_ERROR_NONE;	
}

/**
 * NET_SocketSetAlloc(Uint8 socketset, Uint16 size)
 * Allocates the socketset socketset of size size.
 * Returns NET_ERROR_NONE if successful, else an error.
 */
int NET_SocketSetAlloc(Uint8 socketset, Uint16 size)
{

/* Checks */ 
	
	if(socketset<0 || socketset>=NET_MAX_SOCKETSETS)
	{
		if(*NET.consolereports)
			gr_con_printf("[NET] Error: SocketSet %i invalid\n",socketset);
		return NET_ERROR_INVALIDSOCKETSET;
	}
	if(NET.SocketSet[socketset].status>NET_STATUS_INACTIVE)
	{
		if(*NET.consolereports)
			gr_con_printf("[NET] Error: socketset already active\n");
		return NET_ERROR_SOCKETSETINACTIVE;
	}

/* Make */ 

	NET.SocketSet[socketset].size		= size;
	NET.SocketSet[socketset].SocketSet	= SDLNet_AllocSocketSet( NET.SocketSet[socketset].size );
	if(NET.SocketSet[socketset].SocketSet == NULL)
	{
		if(*NET.consolereports)
			gr_con_printf("[NET] Error: could not allocate socketset\n");
		return NET_ERROR_ALLOCSOCKETSET;
	}

    NET.SocketSet[socketset].status = NET_STATUS_ACTIVE;

	if(*NET.consolereports)
		gr_con_printf("[NET] Socketset(%i) allocated\n",socketset);

	return NET_ERROR_NONE;
}

/**
 * NET_SocketSetFree(Uint8 socketset)
 * Frees the specified socketset.
 * Returns NET_ERROR_NONE if successful, else an error.
 */
int NET_SocketSetFree(Uint8 socketset)
{

/* Checks */ 

//	if(NET.SocketSet[socketset].status<=NET_STATUS_INACTIVE)
//	{
//		if(*NET.consolereports)
//			gr_con_printf("[NET] Error: SocketSet %i inactive\n",socketset);
//		return NET_ERROR_SOCKETSETINACTIVE;
//	}

	if(socketset<0 || socketset>=NET_MAX_SOCKETSETS)
	{
		if(*NET.consolereports)
			gr_con_printf("[NET] Error: SocketSet %i invalid\n",socketset);
		return NET_ERROR_INVALIDSOCKETSET;
	}

/* Destroy */ 
    
	NET.SocketSet[socketset].status = NET_STATUS_INACTIVE;
	SDLNet_FreeSocketSet( NET.SocketSet[socketset].SocketSet );

	if(*NET.consolereports)
		gr_con_printf("[NET] Socketset(%i) freed\n",socketset);

	return NET_ERROR_NONE;
}

/**
 * NET_Send(Uint16 conn,Uint8* s, Uint32 len, Uint8 incsep)
 * Sends a char array s of size len to connection conn. If incsep is not 0, then
 * the connection's separator will be sent immediately after the char array.
 * Returns NET_ERROR_NONE if successful, else an error.
 */
int NET_Send(Uint16 conn,Uint8* s, Uint32 len, Uint8 incsep)
{
	Uint8* s2;

/* Send */

	//add separator, if wanted and if specified
	if(incsep!=0&&NET.Conn[conn].separatorlength>0)
	{
		s2 = malloc(len+NET.Conn[conn].separatorlength);
		memcpy(s2,s,len);
		memcpy(&s2[len],NET.Conn[conn].separator,NET.Conn[conn].separatorlength);
		len+=NET.Conn[conn].separatorlength;
	}
	else
		s2 = s;

	//send
	if(SDLNet_TCP_Send(NET.Conn[conn].socket,s2,len)<(Sint32)len)
	{
		if(NET.Conn[conn].consolereports)
			gr_con_printf("                    FAILED\n");
		return NET_ERROR_SENDING;
	}


	//clean up if needed
	if(incsep!=0&&NET.Conn[conn].separatorlength>0)
		free(s2);		

	NET.transferrate_up[conn] += len;
	NET.transferred_up[conn] += len;
	*NET.totaltransferrate_up += len;
	*NET.totaltransferred_up += len;

	return len;
}

/**
 * NET_GetIncomingMsg()
 *   Receives the bytes from the other peer and adds them
 *   to the incoming message buffer.
 *   Returns the number of bytes added or an error.
 */
int NET_GetIncomingMsg(Uint16 conn)
{
	int r;
	Uint8 s[NET_MAX_MSG_LEN+1];
	int len = NET.Conn[conn].incomingmsglength;

	// get the message
	r = SDLNet_TCP_Recv(NET.Conn[conn].socket,(void*)&s[0],NET_MAX_MSG_LEN);
	if(r<=0)	// there was an error (most likely the peer disconnect)
	{
		NET_Disconnect(conn); // disconnect as well
		return NET_ERROR_DISCONNECTED;
	}
	s[r] = '\0';

	// add the message to the incoming message buffer
	NET.Conn[conn].incomingmsg = realloc(NET.Conn[conn].incomingmsg,len+r+1);
	memcpy(&NET.Conn[conn].incomingmsg[len],&s[0],r);
	NET.Conn[conn].incomingmsg[len+r] = '\0';
	NET.Conn[conn].incomingmsglength = len+r;

	// notify of new message
	NET.Incoming[conn] = NET_STATUS_ACTIVE;

	return len;
}

/**
 * NET_Recv(Uint16 conn, Uint8* s, Uint32 maxlen)
 * Reads incoming bytes (maximum of maxlen) on connection conn to s.
 * If a separator is specified, the returned string will end there.
 * Returns NET_ERROR_NONE if successful, else an error.
 */
int NET_Recv(Uint16 conn, Uint8* s, Uint8 incsep)
{

	Uint32 r;
	int i;
	int len;

/* Checks */ 

	if(conn<NET.maxlistenports || conn>=NET.maxconnections)
	{
		if(*NET.consolereports)
			gr_con_printf("[NET] Error: Connection %i invalid\n",conn);
		return NET_ERROR_INVALIDCONN;
	}
	if(NET.Status[conn]<=NET_STATUS_INACTIVE)
	{
		if(*NET.consolereports)
			gr_con_printf("[NET] Error: Connection %i not active\n",conn);
		return NET_ERROR_CONNINACTIVE;
	}

/* More checks */

	// change status to inactive if there are no messages and return NET_ERROR_NONEWMESSAGE
	if(NET.Conn[conn].incomingmsglength==0)
	{
		if(NET.Incoming[conn] == NET_STATUS_ACTIVE)
			NET.Incoming[conn] = NET_STATUS_INACTIVE;
		return NET_ERROR_NONEWMESSAGE;
	}

	//search the end of the message
	if(NET.Conn[conn].separatorlength>0)
	{
		// look for the first separator
		for(r=0;r+NET.Conn[conn].separatorlength<=NET.Conn[conn].incomingmsglength;r++)
		{
			// check if the separator is here
			for(i=0; i<NET.Conn[conn].separatorlength; i++)
				if(NET.Conn[conn].incomingmsg[r+i]!=NET.Conn[conn].separator[i])
					break;
			if(i==NET.Conn[conn].separatorlength)
				break; // end of message
		}
		// check if separator was found
		if(r+NET.Conn[conn].separatorlength>NET.Conn[conn].incomingmsglength) // if not
		{
			if(NET.Incoming[conn] == NET_STATUS_ACTIVE)
				NET.Incoming[conn] = NET_STATUS_INACTIVE;
			return NET_ERROR_UNFINISHEDMESSAGE; // unfinished message
		}
		len = r;
	}
	else
		len = NET.Conn[conn].incomingmsglength;

/* Receive */
	
	//move the message
	memcpy(&s[0],&NET.Conn[conn].incomingmsg[0],len+NET.Conn[conn].separatorlength*(incsep!=0));
//	s[len] = '\0';
	NET.Conn[conn].incomingmsglength-=len+NET.Conn[conn].separatorlength;
	if(NET.Conn[conn].incomingmsglength>0)
		memcpy(&NET.Conn[conn].incomingmsg[0],&NET.Conn[conn].incomingmsg[len+NET.Conn[conn].separatorlength],NET.Conn[conn].incomingmsglength);
	NET.Conn[conn].incomingmsg = realloc(NET.Conn[conn].incomingmsg,NET.Conn[conn].incomingmsglength+1);
	NET.Conn[conn].incomingmsg[NET.Conn[conn].incomingmsglength] = '\0';

	// change status to inactive if no more messages
	if(NET.Conn[conn].incomingmsglength==0)
		if(NET.Incoming[conn] == NET_STATUS_ACTIVE)
			NET.Incoming[conn] = NET_STATUS_INACTIVE;

	len+=NET.Conn[conn].separatorlength;

	// DUM
	NET.transferrate_down[conn] += len;
	NET.transferred_down[conn] += len;
	*NET.totaltransferrate_down += len;
	*NET.totaltransferred_down += len;

	return len-NET.Conn[conn].separatorlength;
}

/**
 * NET_RecvVar(Uint16 conn, Uint8* s, Uint32 expectedlen)
 * Reads incoming bytes (maximum of maxlen) on connection conn to s 
 * If the incomingmessage buffer is too small, an error is returned.
 * If a separator is specified, it will be checked if there is a message
 * waiting with the specified length. If there isn't, an error is returned.
 * If there is, it will be written to s.
 * Returns NET_ERROR_NONE if successful, else an error.
 */
int NET_RecvVar(Uint16 conn, Uint8* s, Uint32 expectedlen, Uint8 incsep)
{

	int i;

/* More checks */

	if(s==NULL)
	{
		if(*NET.consolereports)
			gr_con_printf("[NET] Error: NULL pointer exception on %i\n",conn);
		return NET_ERROR_NULLPOINTER;
	}
	if(expectedlen>NET.Conn[conn].incomingmsglength-NET.Conn[conn].separatorlength)
	{
		return NET_ERROR_UNFINISHEDMESSAGE;
	}

	// change status to inactive if there are no messages and return NET_ERROR_NONEWMESSAGE
	if(NET.Conn[conn].incomingmsglength==0)
	{
		if(NET.Incoming[conn] == NET_STATUS_ACTIVE)
			NET.Incoming[conn] = NET_STATUS_INACTIVE;
		if(*NET.consolereports)
			gr_con_printf("[NET] Error: No new message on %i\n",conn);
		return NET_ERROR_NONEWMESSAGE;
	}

	// check if size matches
	if(NET.Conn[conn].separatorlength>0) // a separator is specified
	{
		// check if the separator is at the expected position
		for(i=0; i<NET.Conn[conn].separatorlength; i++)
			if(NET.Conn[conn].incomingmsg[expectedlen+i]!=NET.Conn[conn].separator[i])
				break;
		if(i!=NET.Conn[conn].separatorlength) // it isn't
		{
			if(*NET.consolereports)
				gr_con_printf("[NET] Error: Message size mismatch on %i\n",conn);
			return NET_ERROR_SIZEMISMATCH;
		}
	}
	
/* Receive */

	//move the message
	memcpy(&s[0],&NET.Conn[conn].incomingmsg[0],expectedlen+NET.Conn[conn].separatorlength*(incsep!=0));
//	s[expectedlen] = '\0';

	NET.Conn[conn].incomingmsglength-=expectedlen+NET.Conn[conn].separatorlength;
	if(NET.Conn[conn].incomingmsglength>0)
		memcpy(&NET.Conn[conn].incomingmsg[0],&NET.Conn[conn].incomingmsg[expectedlen+NET.Conn[conn].separatorlength],NET.Conn[conn].incomingmsglength);
	NET.Conn[conn].incomingmsg = realloc(NET.Conn[conn].incomingmsg,NET.Conn[conn].incomingmsglength+1);
	NET.Conn[conn].incomingmsg[NET.Conn[conn].incomingmsglength] = '\0';

	// change status to inactive if no more messages
	if(NET.Conn[conn].incomingmsglength==0)
		if(NET.Incoming[conn] == NET_STATUS_ACTIVE)
			NET.Incoming[conn] = NET_STATUS_INACTIVE;

	expectedlen+=NET.Conn[conn].separatorlength;

	// DUM
	NET.transferrate_down[conn] += expectedlen;
	NET.transferred_down[conn] += expectedlen;
	*NET.totaltransferrate_down += expectedlen;
	*NET.totaltransferred_down += expectedlen;

	return expectedlen-NET.Conn[conn].separatorlength;
}

/**
 * NET_Listen(Uint8 set,Uint16 port, Sint8 consolereports)
 * Start listening on port, add the serversocket to socketset set
 * and specify whether consolereports are wanted or not.
 * Returns NET_ERROR_NONE if successful, else an error.
 */
int NET_Listen(Uint8 set,Uint16 port, Sint8 consolereports)
{

/* Checks */

	Uint16 conn;

	if(set<0 || set>=NET_MAX_SOCKETSETS)
	{
		if(*NET.consolereports)
			gr_con_printf("[NET] Error: SocketSet %i invalid\n",set);
		return NET_ERROR_INVALIDSOCKETSET;
	}
	if(NET.SocketSet[set].status<=NET_STATUS_INACTIVE)
	{
		if(*NET.consolereports)
			gr_con_printf("[NET] Error: SocketSet %i invalid\n",set);
		return NET_ERROR_SOCKETSETINACTIVE;
	}
	if(port<0||port>65535)
	{
		if(*NET.consolereports)
			gr_con_printf("[NET] Error: SocketSet %i invalid\n",set);
		return NET_ERROR_INVALIDPORTNUMBER;
	}
	if(consolereports!=0)
		consolereports = 1;

/* Find Empty Place */

	for(conn=0; conn<NET.maxlistenports; conn++)
		if(NET.Status[conn]<=NET_STATUS_INACTIVE)
			break;
	if(conn>=NET.maxlistenports)
	{
		if(*NET.consolereports)
			gr_con_printf("[NET] Error: too many listenports\n");
		return NET_ERROR_TOOMANYLISTENPORTS;
	}

/* Open */

	// resolve host
	if( SDLNet_ResolveHost( &NET.Conn[conn].address, NULL, port) == -1 )
	{
		if(*NET.consolereports)
			gr_con_printf("[NET] Error: could not resolve host\n");
		return NET_ERROR_RESOLVINGHOST;
	}
	// open connection
	NET.Conn[conn].socket = SDLNet_TCP_Open( &NET.Conn[conn].address );
	if( NET.Conn[conn].socket == NULL )
	{
		if(*NET.consolereports)
			gr_con_printf("[NET] Error: could not listen on connection\n");
		return NET_ERROR_LISTENINGONCONN;
	}

/* Socket Set */

	// add socket to socketset
	if( SDLNet_TCP_AddSocket(NET.SocketSet[set].SocketSet,NET.Conn[conn].socket) == -1)
	{
		if(*NET.consolereports)
			gr_con_printf("[NET] Error: could not add socket\n");
		return NET_ERROR_ADDINGSOCKET;
	}
	NET.Conn[conn].socketset_id = set;	

/* Data */

	// prepare connection to be used
	net_prepareconn(conn);

	NET.Conn[conn].consolereports = consolereports;
	NET.Status[conn] = NET_STATUS_ESTABLISHED;

	(*NET.activelistenports)++;

	if(NET.Conn[conn].consolereports)
	{
		gr_con_printf("[NET] Listenport(%i): %i\n",conn,(int)port);
	}

	return conn;
}

/**
 * NET_Connect(Uint8 set, Uint8* hostname, Uint16 port, Sint8 consolereports)
 * Connect to hostname:port, add the socket to socketset set
 * and specify whether consolereports are wanted or not.
 * Returns NET_ERROR_NONE if successful, else an error.
 */
int NET_Connect(Uint8 set, Uint8* hostname, Uint16 port, Sint8 consolereports)
{

	Uint16 conn;

/* Checks */

	if(set<0 || set>=NET_MAX_SOCKETSETS)
	{
		if(*NET.consolereports)
			gr_con_printf("[NET] Error: SocketSet %i invalid\n",set);
		return NET_ERROR_INVALIDSOCKETSET;
	}
	if(NET.SocketSet[set].status<=NET_STATUS_INACTIVE)
	{
		if(*NET.consolereports)
			gr_con_printf("[NET] Error: socketset not active: %i\n",set);
		return NET_ERROR_SOCKETSETINACTIVE;
	}
	if(port<0||port>65535)
	{
		if(*NET.consolereports)
			gr_con_printf("[NET] Error: SocketSet %i invalid\n",set);
		return NET_ERROR_INVALIDPORTNUMBER;
	}
	if(consolereports!=0)
		consolereports = 1;

/* Find Empty Place */

	for(conn=NET.maxlistenports; conn<NET.maxconnections; conn++)
		if(NET.Status[conn]<=NET_STATUS_INACTIVE)
			break;
	if(conn>=NET.maxconnections)
	{
		if(*NET.consolereports)
			gr_con_printf("[NET] Error: too many connections\n");
		return NET_ERROR_TOOMANYCONNS;
	}

/* Open */

	if( SDLNet_ResolveHost( &NET.Conn[conn].address, hostname, port) == -1 )
	{
		if(*NET.consolereports)
			gr_con_printf("[NET] Error: could not resolve %s:%i on %i\n",hostname,port,conn);
		return NET_ERROR_RESOLVINGHOST;
	}
	NET.Conn[conn].socket = SDLNet_TCP_Open( &NET.Conn[conn].address );
	if( NET.Conn[conn].socket == NULL )
	{
		if(*NET.consolereports)
			gr_con_printf("[NET] Error: could not connect to %s:%i on %i\n",hostname,port,conn);
		return NET_ERROR_CONNECTING;
	}

/* Socket Set */

	if( SDLNet_TCP_AddSocket(NET.SocketSet[set].SocketSet,NET.Conn[conn].socket) == -1)
	{
		if(*NET.consolereports)
			gr_con_printf("[NET] Error: could not add %s:%i\n to socket %i on %i\n",hostname,port,set,conn);
		return NET_ERROR_ADDINGSOCKET;
	}

/* Data */

	NET.Conn[conn].socketset_id = set;	

	// prepare connection to be used
	net_prepareconn(conn);

//	NET.Conn[conn].timeoutcounter = 0;
	NET.Conn[conn].consolereports = consolereports;

	NET.Status[conn] = NET_STATUS_ESTABLISHING;

	(*NET.activeconnections)++;

	if(NET.Conn[conn].consolereports)
		gr_con_printf("[NET] Connection(%i): %s:%i\n",conn,hostname,port);

	return conn;
}

/**
 * NET_Disconnect(Uint16 conn)
 * Connect to from connection conn.
 * Returns NET_ERROR_NONE if successful, else an error.
 */
int NET_Disconnect(Uint16 conn)
{
	Uint8* s;

/* Checks */

	if(conn<0 || conn>=NET.maxconnections)
	{
		if(*NET.consolereports)
			gr_con_printf("[NET] Error: Connection %i invalid\n",conn);
		return NET_ERROR_INVALIDCONN;
	}
	if(NET.Status[conn]<=NET_STATUS_INACTIVE)
	{
		if(*NET.consolereports)
			gr_con_printf("[NET] Error: Connection %i not active\n",conn);
		return NET_ERROR_CONNINACTIVE;
	}

/* Socket Set */

	if( SDLNet_TCP_DelSocket(NET.SocketSet[NET.Conn[conn].socketset_id].SocketSet,NET.Conn[conn].socket) == -1)
	{
		if(*NET.consolereports)
			gr_con_printf("[NET] Error: could not delete connection %i from socket\n",conn);
		return NET_ERROR_DELETINGSOCKET;
	}
	NET.Conn[conn].socketset_id = -1;	

/* Close */

	SDLNet_TCP_Close( NET.Conn[conn].socket );

/* Data */

//	NET.Conn[conn].timeoutcounter = 0;

	NET.Status[conn] = NET_STATUS_DISCONNECTING;

	if(conn<NET.maxlistenports)
	{
		(*NET.activelistenports)--;
		if(NET.Conn[conn].consolereports)
		{
			gr_con_printf("[NET] Listenport(%i) closed\n",conn);
		}
	}
	else
	{
		(*NET.activeconnections)--;
		if(NET.Conn[conn].consolereports)
		{
			s = (Uint8*)SDLNet_ResolveIP(&NET.Conn[conn].address);
			if(s) // if the hostname was successfully resolved
				gr_con_printf("[NET] Connection(%i) disconnected from %s\n",conn,s);
			else // else
				gr_con_printf("[NET] Connection(%i) disconnected\n",conn);
		}
	}

	// release the connection for later use
	net_releaseconn(conn);

	return NET_ERROR_NONE;
}

/**
 * NET_Accept(Uint8 set, Uint16 inconn, TCPsocket socket)
 * Accept the connection made to inconn, which generated
 * TCPsocket socket, and add it to socketset set.
 * Returns NET_ERROR_NONE if successful, else an error.
 */
int NET_Accept(Uint8 set, Uint16 inconn, TCPsocket socket)
{

/* Checks */

	Uint16 conn;
	IPaddress * ipa; // temp
	Uint8* s; //hostname;

	if(set<0 || set>=NET_MAX_SOCKETSETS)
	{
		if(*NET.consolereports)
			gr_con_printf("[NET] Error: SocketSet %i invalid\n",set);
		return NET_ERROR_INVALIDSOCKETSET;
	}
	if(NET.SocketSet[set].status<=NET_STATUS_INACTIVE)
	{
		if(*NET.consolereports)
			gr_con_printf("[NET] Error: socketset not active: %i\n",set);
		return NET_ERROR_SOCKETSETINACTIVE;
	}
	if(socket==NULL)
	{
		if(*NET.consolereports)
			gr_con_printf("[NET] Error: invalid socket\n");
		return NET_ERROR_INVALIDSOCKET;
	}

/* Find Empty Place */

	for(conn=NET.maxlistenports; conn<NET.maxconnections; conn++)
		if(NET.Status[conn]<=NET_STATUS_INACTIVE)
			break;
	if(conn>=NET.maxconnections)
	{
		if(*NET.consolereports)
			gr_con_printf("[NET] Error: too many connections\n");
		return NET_ERROR_TOOMANYCONNS;
	}

/* Open */

	net_prepareconn(conn);
	NET.Conn[conn].socket = socket;

/* Socket Set */

	if( SDLNet_TCP_AddSocket(NET.SocketSet[set].SocketSet,NET.Conn[conn].socket) == -1)
	{
		if(*NET.consolereports)
			gr_con_printf("[NET] Error: could not add socket\n");
		return NET_ERROR_ADDINGSOCKET;
	}
	NET.Conn[conn].socketset_id = set;	

/* Data */

	ipa = SDLNet_TCP_GetPeerAddress(NET.Conn[conn].socket);
	NET.Conn[conn].address = *ipa;
	
//	NET.Conn[conn].timeoutcounter = 0;
	NET.Conn[conn].consolereports = NET.Conn[inconn].consolereports;

	NET.Status[conn] = NET_STATUS_ESTABLISHING;

	(*NET.activeconnections)++;

	if(NET.Conn[conn].consolereports)
	{
		s = (Uint8*)SDLNet_ResolveIP(&NET.Conn[conn].address);
		if(s) // if the hostname was successfully resolved
			gr_con_printf("[NET] Connection(%i): %s:%i\n",conn,s,ipa->port);
		else // else
			gr_con_printf("[NET] Connection(%i)\n",conn);
	}

	return conn;
}

/**
 * NET_Separator(Uint16 conn, Uint8* s, Uint8 len, Uint8 incsep)
 * Set the separator on connection conn to s with length len.
 * Specify whether the separator should be included when a
 * message is returned.
 * If s is NULL, there will be no separator and the current one
 * will be cleared
 * Returns NET_ERROR_NONE if successful, else an error.
 */
int NET_Separator(Uint16 conn, Uint8* s, Uint8 len)
{

/* Checks */

	if(conn<0 || conn>=NET.maxconnections)
	{
		if(*NET.consolereports)
			gr_con_printf("[NET] Error: Connection %i invalid\n",conn);
		return NET_ERROR_INVALIDCONN;
	}
	if(NET.Status[conn]<=NET_STATUS_INACTIVE)
	{
		if(*NET.consolereports)
			gr_con_printf("[NET] Error: Connection %i not active\n",conn);
		return NET_ERROR_CONNINACTIVE;
	}

/* Separator */

	if(s==NULL&&NET.Conn[conn].separatorlength==0) // clear the separator, if not done already
	{
		NET.Conn[conn].separatorlength = 0;
		free(NET.Conn[conn].separator);
		if(*NET.consolereports)
			gr_con_printf("[NET] Separator(%i) cleared\n",conn);
	}
	else // set the separator
	{
		if(NET.Conn[conn].separatorlength>0)
			free(NET.Conn[conn].separator);
		NET.Conn[conn].separator = malloc(len+1);
		memcpy(NET.Conn[conn].separator,s,len);
		NET.Conn[conn].separator[len] = '\0';
		NET.Conn[conn].separatorlength = len;
		if(*NET.consolereports)
			gr_con_printf("[NET] Separator(%i): '%s' (%i)\n",conn,NET.Conn[conn].separator,NET.Conn[conn].separatorlength);
	}
	return NET_ERROR_NONE;
}

/********************************************************************\
*                                                                    *
* Fenix Functions                                            Sandman *
*                                                                    *
\********************************************************************/

static int NET_ABOUT (INSTANCE * my, int * params)
{
	gr_con_printf("[NET] Network.DLL %s,\n",NETversionreply);
	gr_con_printf("      created by Sandman\n");

	return NETversion;
}

static int NET_VERSION (INSTANCE * my, int * params)
{
	int r = string_new(NETversionreply);
	string_use(r);
	return r;
}

static int NET_INTVERSION (INSTANCE * my, int * params)
{
	return NETversion;
}

static int NET_INIT (INSTANCE * my, int * params)
{//NET_Init(timeout,pingtimeout,maxconnections,numberoflistenports,incomingpointer,statuspointer,activeconnectionspointer,activelistenconnectionspointer,consolereportspointer,returnnetdllcommandspointer);

	int i;

/* Checks */

	if(NET.status==NET_STATUS_ACTIVE)
	{
		if(*NET.consolereports)
			gr_con_printf("[NET] Error: Network.DLL already initialized\n");
		return NET_ERROR_ALREADYINIT;
	}
	if( SDLNet_Init() == -1 )
	{
		if(*NET.consolereports)
			gr_con_printf("[NET] Error: could not initialize Network.DLL\n");
		return NET_ERROR_INITIALIZATION;
	}

/* Params */

	NET.timeout				= params[0];

	NET.maxconnections		= (Uint16)params[1];
	NET.maxlistenports		= (Uint16)params[2];

	if(NET.maxconnections<1)
	{
		if(*NET.consolereports)
			gr_con_printf("[NET] Error: too few connections\n");
		return NET_ERROR_TOOFEWCONNS;
	}
	if(NET.maxlistenports>=NET.maxconnections)
	{
		if(*NET.consolereports)
			gr_con_printf("[NET] Error: too many listenports\n");
		return NET_ERROR_TOOMANYLISTENPORTS;
	}

	NET.sendportbegin			= (Uint16)0;
	NET.sendportend				= (Uint16)65536;

	NET.Incoming				= (Sint32*) params[3];
	NET.Status					= (Sint32*) params[4];
	NET.activeconnections		= (Uint16*) params[5];
	NET.activelistenports		= (Uint16*) params[6];
	NET.consolereports			= (Uint8*)  params[7];
	NET.returnnetdllcommands	= (Uint8*)  params[8];

	if((*NET.consolereports)!=0)
		*(NET.consolereports) = 1;

/* Preparation */

	for(i=0; i<NET_MAX_SOCKETSETS;i++)
		NET.SocketSet[i].status = NET_STATUS_INACTIVE;

	NET.Conn	= malloc(NET.maxconnections*sizeof(_con  ));
	NET.ExtConn	= malloc(NET.maxconnections*sizeof(Uint16));

/* Active */

	NET.status = NET_STATUS_ACTIVE;
	gr_con_printf("[NET] Network.DLL %s initialized\n",NETversion_string);

/* SocketSet */
	
	NET_SocketSetAlloc(0,NET.maxconnections);

	return NET_ERROR_NONE;
}

static int NET_DUMINIT (INSTANCE * my, int * params)
{//NET_DUMInit( alot of pointers );

	NET.totaltransferrate_up	= (Sint32*) params[0];
	NET.totaltransferrate_down	= (Sint32*) params[1];
	NET.totaltransferred_up		= (Sint32*) params[2];
	NET.totaltransferred_down	= (Sint32*) params[3];

	NET.transferrate_up			= (Sint32*) params[4];
	NET.transferrate_down		= (Sint32*) params[5];
	NET.transferred_up			= (Sint32*) params[6];
	NET.transferred_down		= (Sint32*) params[7];

	return NET_ERROR_NONE;
}

static int NET_LOOP (INSTANCE * my, int * params)
{
	Uint8 set=0;
	Uint16 conn=0;
	int res=0;
	int len=0;

/* Checks */

//	if(NET.status == NET_STATUS_INACTIVE)
//		return NET_ERROR_NOTINITIALIZED;

/* Check for incoming connections */
	for(conn=0; conn<NET.maxlistenports; conn++)
	{
		if(NET.Status[conn]>NET_STATUS_INACTIVE)
		{
			NET.tempsocket = SDLNet_TCP_Accept(NET.Conn[conn].socket);
			if(NET.tempsocket!=NULL)
			{
				NET_Accept(0,conn,NET.tempsocket);
			}
		}
	}

/* Sort out the status of the connections and reset the transfer rates */
	for(conn=0; conn<NET.maxconnections; conn++)
	{
		NET.transferrate_up[conn]	= 0;
		NET.transferrate_down[conn]	= 0;
		switch(NET.Status[conn])
		{
			case NET_STATUS_DISCONNECTED:
				NET.Incoming[conn] = NET_STATUS_INACTIVE;
				break;
			case NET_STATUS_DISCONNECTING:
				NET.Status[conn] = NET_STATUS_DISCONNECTED;
				NET.Incoming[conn] = NET_STATUS_DISCONNECTED;
				break;
			case NET_STATUS_ESTABLISHED:
				NET.Incoming[conn] = NET_STATUS_INACTIVE;
				break;
			case NET_STATUS_ESTABLISHING:
				NET.Incoming[conn] = NET_STATUS_ESTABLISHED;
				NET.Status[conn] = NET_STATUS_ESTABLISHED;
				break;
		}
	}

/* Activity check */

//	for(set=0; set<=NET_MAX_SOCKETSETS; set++)
	{
		if(NET.SocketSet[set].status>NET_STATUS_INACTIVE)
		{
			if(SDLNet_CheckSockets( NET.SocketSet[set].SocketSet, NET.timeout )>NET_STATUS_INACTIVE)
			{
				for(conn=NET.maxlistenports; conn<NET.maxconnections; conn++)
				{
					if(NET.Status[conn]>NET_STATUS_INACTIVE)
					{
						if(SDLNet_SocketReady(NET.Conn[conn].socket)>NET_STATUS_INACTIVE)
						{
							if(NET.Incoming[conn]==NET_STATUS_INACTIVE||NET.Incoming[conn]==NET_STATUS_ACTIVE)
								NET_GetIncomingMsg(conn);
						}//SocketReady (Incoming message)
					}//IF NET.Status[conn]>NET_STATUS_INACTIVE
				}//FOR conn
			}//active socketset
		}//active set
	}//FOR set
	
	return res;
}

static int NET_CONNECT (INSTANCE * my, int * params)
{// NET_Open( hostname , port );
	int r;
	Uint8* s = (Uint8*)string_get(params[0]);
	r = NET_Connect(0,s,(Uint16)params[1],NET_CONSOLEREPORTS_NO);
	string_discard(params[0]);
	return r;
}

static int NET_CONNECT2 (INSTANCE * my, int * params)
{// NET_Open( hostname , port , consolereports on/off );
	int r;
	Uint8* s = (Uint8*)string_get(params[0]);
	r = NET_Connect(0,s,(Uint16)params[1],(Sint8)params[2]);
	string_discard(params[0]);
	return r;
}

static int NET_DISCONNECT (INSTANCE * my, int * params)
{// NET_Close( conn );
	return NET_Disconnect((Uint16)params[0]);
}

static int NET_DISCONNECTALL (INSTANCE * my, int * params)
{
	int i;

	for(i=0; i<NET.maxconnections; i++)
		NET_Disconnect((Uint16)i);

	return NET_ERROR_NONE;
}

static int NET_SEND (INSTANCE * my, int * params)
{// NET_Send( conn , messagestring );
	Uint32 r;
	Uint8* s;

/* Checks */ 

	if(params[0]<NET.maxlistenports || params[0]>=NET.maxconnections)
	{
		if(*NET.consolereports)
			gr_con_printf("[NET] Error: Connection %i invalid\n",params[0]);
		return NET_ERROR_INVALIDCONN;
	}
	if(NET.Status[params[0]]<=NET_STATUS_INACTIVE)
	{
		if(*NET.consolereports)
			gr_con_printf("[NET] Error: Connection %i not active\n",params[0]);
		return NET_ERROR_CONNINACTIVE;
	}
	s = (Uint8*)string_get(params[1]);
	r = strlen(s);
	if(r<=0)
	{
		if(*NET.consolereports)
			gr_con_printf("[NET] Error: Message too short (%i)\n",params[0]);
		return NET_ERROR_MESSAGETOOSHORT;
	}

/* Action */

	if(NET.Conn[params[0]].consolereports)
		gr_con_printf("[NET] OutgoingM(%i): '%s', %i (%i)\n",params[0],s,r,r+NET.Conn[params[0]].separatorlength);

	r = NET_Send((Uint16)params[0],s,r,1);

	string_discard(params[1]);
	return r;
}

static int NET_SEND2 (INSTANCE * my, int * params)
{// NET_Send( conn , messagestring , includeseparator );
	Uint32 r;
	Uint8* s;

/* Checks */ 

	if(params[0]<NET.maxlistenports || params[0]>=NET.maxconnections)
	{
		if(*NET.consolereports)
			gr_con_printf("[NET] Error: Connection %i invalid\n",params[0]);
		return NET_ERROR_INVALIDCONN;
	}
	if(NET.Status[params[0]]<=NET_STATUS_INACTIVE)
	{
		if(*NET.consolereports)
			gr_con_printf("[NET] Error: Connection %i not active\n",params[0]);
		return NET_ERROR_CONNINACTIVE;
	}
	s = (Uint8*)string_get(params[1]);
	r = strlen(s);
	if(r<=0)
	{
		if(*NET.consolereports)
			gr_con_printf("[NET] Error: Message too short (%i)\n",params[0]);
		return NET_ERROR_MESSAGETOOSHORT;
	}

/* Action */

	if(NET.Conn[params[0]].consolereports)
		gr_con_printf("[NET] OutgoingM(%i): '%s', %i (%i)\n",params[0],s,r,r+(NET.Conn[params[0]].separatorlength*(params[2]!=0)));

	r=NET_Send((Uint16)params[0],s,r,(Uint8)params[2]);

	string_discard(params[1]);
	return r;
}


static int NET_SENDRN (INSTANCE * my, int * params)
{// NET_SendRN( conn , messagestring );

	Uint8* s2;
	Uint8* s;
	Uint32 r;

/* Checks */ 

	if(params[0]<NET.maxlistenports || params[0]>=NET.maxconnections)
	{
		if(*NET.consolereports)
			gr_con_printf("[NET] Error: Connection %i invalid\n",params[0]);
		return NET_ERROR_INVALIDCONN;
	}
	if(NET.Status[params[0]]<=NET_STATUS_INACTIVE)
	{
		if(*NET.consolereports)
			gr_con_printf("[NET] Error: Connection %i not active\n",params[0]);
		return NET_ERROR_CONNINACTIVE;
	}
	s = (Uint8*)string_get(params[1]);
	r = strlen(s);
	if(r<=0)
	{
		if(*NET.consolereports)
			gr_con_printf("[NET] Error: Message too short (%i)\n",params[0]);
		return NET_ERROR_MESSAGETOOSHORT;
	}

/* Action */

	s2 = malloc(r+3);
	strcpy(s2,s);
	s2[r  ] = '\r';
	s2[r+1] = '\n';
	s2[r+2] = '\0';

	if(NET.Conn[params[0]].consolereports)
		gr_con_printf("[NET] OutgoingM(%i): '%s', %i (%i)\n",params[0],s,r,r+2);

	r =  NET_Send((Uint16)params[0],s2,r+2,0);

	free(s2);
	string_discard(params[1]);
	return r;
}

static int NET_SENDVAR (INSTANCE * my, int * params)
{// NET_SendVar( conn , messagepointer , messagelength );

/* Checks */ 

	if(params[0]<NET.maxlistenports || params[0]>=NET.maxconnections)
	{
		if(*NET.consolereports)
			gr_con_printf("[NET] Error: Connection %i invalid\n",params[0]);
		return NET_ERROR_INVALIDCONN;
	}
	if(NET.Status[params[0]]<=NET_STATUS_INACTIVE)
	{
		if(*NET.consolereports)
			gr_con_printf("[NET] Error: Connection %i not active\n",params[0]);
		return NET_ERROR_CONNINACTIVE;
	}
	if(params[2]<=0)
	{
		if(*NET.consolereports)
			gr_con_printf("[NET] Error: Message too short (%i)\n",params[0]);
		return NET_ERROR_MESSAGETOOSHORT;
	}

/* Action */

	if(NET.Conn[params[0]].consolereports)
		gr_con_printf("[NET] OutgoingV(%i): %i (%i)\n",params[0],params[2],params[2]);

	return NET_Send((Uint16)params[0],(Uint8*)params[1],(Uint32)params[2],1);
}

static int NET_SENDVAR2 (INSTANCE * my, int * params)
{// NET_SendVar( conn , messagepointer , messagelength , includeseparator );

/* Checks */ 

	if(params[0]<NET.maxlistenports || params[0]>=NET.maxconnections)
	{
		if(*NET.consolereports)
			gr_con_printf("[NET] Error: Connection %i invalid\n",params[0]);
		return NET_ERROR_INVALIDCONN;
	}
	if(NET.Status[params[0]]<=NET_STATUS_INACTIVE)
	{
		if(*NET.consolereports)
			gr_con_printf("[NET] Error: Connection %i not active\n",params[0]);
		return NET_ERROR_CONNINACTIVE;
	}
	if(params[2]<=0)
	{
		if(*NET.consolereports)
			gr_con_printf("[NET] Error: Message too short (%i)\n",params[0]);
		return NET_ERROR_MESSAGETOOSHORT;
	}

/* Action */

	if(NET.Conn[params[0]].consolereports)
		gr_con_printf("[NET] OutgoingV(%i): %i (%i)\n",params[0],params[2],params[2]+(NET.Conn[params[0]].separatorlength*(params[3]!=0)));

	return NET_Send((Uint16)params[0],(Uint8*)params[1],(Uint32)params[2],(Uint8)params[3]);
}

static int NET_DLLSEND (INSTANCE * my, int * params)
{// NET_DLLSend( conn , messagepointer , messagelength , includeseparator );
	return NET_Send((Uint16)params[0],(Uint8*)params[1],(Uint32)params[2],(Uint8)params[3]);
}

static int NET_RECV (INSTANCE * my, int * params)
{// NET_Recv( conn );
	int r,t;
	Uint8 msg[NET_MAX_MSG_LEN];

/* Checks */ 

	if(params[0]<NET.maxlistenports || params[0]>=NET.maxconnections)
	{
		if(*NET.consolereports)
			gr_con_printf("[NET] Error: Connection %i invalid\n",params[0]);
		return NET_ERROR_INVALIDCONN;
	}
	if(NET.Status[params[0]]<=NET_STATUS_INACTIVE)
	{
		if(*NET.consolereports)
			gr_con_printf("[NET] Error: Connection %i not active\n",params[0]);
		return NET_ERROR_CONNINACTIVE;
	}

/* Action */

	t = NET_Recv((Uint16)params[0],msg,0);
	if(t<0)
		r = string_new("");
	else
	{
		msg[t] = '\0';
		r = string_new(msg);
		if(NET.Conn[params[0]].consolereports)
			gr_con_printf("[NET] IncomingM(%i): '%s', %i (%i)\n",params[0],msg,t,t+NET.Conn[params[0]].separatorlength);
	}
	string_use(r);
	return r;
}

static int NET_RECV2 (INSTANCE * my, int * params)
{// NET_Recv( conn , includeseparator );
	int r,t;
	Uint8 msg[NET_MAX_MSG_LEN];

/* Checks */ 

	if(params[0]<NET.maxlistenports || params[0]>=NET.maxconnections)
	{
		if(*NET.consolereports)
			gr_con_printf("[NET] Error: Connection %i invalid\n",params[0]);
		return NET_ERROR_INVALIDCONN;
	}
	if(NET.Status[params[0]]<=NET_STATUS_INACTIVE)
	{
		if(*NET.consolereports)
			gr_con_printf("[NET] Error: Connection %i not active\n",params[0]);
		return NET_ERROR_CONNINACTIVE;
	}

/* Action */

	t = NET_Recv((Uint16)params[0],msg,(Uint8)(params[1]!=0));
	if(t<0)
	{
		if(NET.Conn[params[0]].consolereports)
			gr_con_printf("[NET] IncomingM(%i): failed: %i\n",params[0],t);
		r = string_new("");
	}
	else
	{
		msg[t] = '\0';
		r = string_new(msg);
		if(NET.Conn[params[0]].consolereports)
			gr_con_printf("[NET] IncomingM(%i): '%s', %i (%i)\n",params[0],msg,t,t+NET.Conn[params[0]].separatorlength);
	}
	string_use(r);
	return r;
}

static int NET_RECVVAR (INSTANCE * my, int * params)
{// NET_RecvVar( conn , messagepointer , messagelength );
	Uint32 r;
/* Checks */ 

	if(params[0]<NET.maxlistenports || params[0]>=NET.maxconnections)
	{
		if(*NET.consolereports)
			gr_con_printf("[NET] Error: Connection %i invalid\n",params[0]);
		return NET_ERROR_INVALIDCONN;
	}
	if(NET.Status[params[0]]<=NET_STATUS_INACTIVE)
	{
		if(*NET.consolereports)
			gr_con_printf("[NET] Error: Connection %i not active\n",params[0]);
		return NET_ERROR_CONNINACTIVE;
	}

/* Action */

	r = NET_RecvVar((Uint16)params[0],(Uint8*)params[1],(Uint32)params[2],0);
	if(r>=0)
	{
		if(NET.Conn[params[0]].consolereports)
			gr_con_printf("[NET] IncomingV(%i): %i (%i)\n",params[0],r,r+NET.Conn[params[0]].separatorlength);
	}
	else
		if(NET.Conn[params[0]].consolereports)
			gr_con_printf("[NET] IncomingV(%i): %i failed: %i\n",params[0],params[2],r);
	return r;
}

static int NET_RECVVAR2 (INSTANCE * my, int * params)
{// NET_RecvVar( conn , messagepointer , messagelength , includeseparator);
	Uint32 r;
/* Checks */ 

	if(params[0]<NET.maxlistenports || params[0]>=NET.maxconnections)
	{
		if(*NET.consolereports)
			gr_con_printf("[NET] Error: Connection %i invalid\n",params[0]);
		return NET_ERROR_INVALIDCONN;
	}
	if(NET.Status[params[0]]<=NET_STATUS_INACTIVE)
	{
		if(*NET.consolereports)
			gr_con_printf("[NET] Error: Connection %i not active\n",params[0]);
		return NET_ERROR_CONNINACTIVE;
	}

/* Action */

	r = NET_RecvVar((Uint16)params[0],(Uint8*)params[1],(Uint32)params[2],(Uint8)(params[3]!=0));
	if(r>=0)
		if(NET.Conn[params[0]].consolereports)
			gr_con_printf("[NET] IncomingV(%i): %i (%i)\n",params[0],r,r+NET.Conn[params[0]].separatorlength);
	return r;
}

static int NET_DLLRECVVAR (INSTANCE * my, int * params)
{
	return NET_RecvVar((Uint16)params[0],(Uint8*)params[1],(Uint32)params[2],(Uint8)(params[3]!=0));
}
static int NET_QUIT (INSTANCE * my, int * params)
{
	int i;

	for(i=0; i<NET.maxconnections; i++)
		NET_Disconnect((Uint16)i);
	free(NET.Conn);
	free(NET.ExtConn);
	NET.Conn = NULL;
	NET.ExtConn = NULL;

	NET.status = NET_STATUS_INACTIVE;

	SDLNet_Quit();

	gr_con_printf("[NET] Network.DLL %s deinitialized\n",NETversion_string);
	return NET_ERROR_NONE;
}

int NET_GETERROR_OLD ()
{
	Uint8*a=SDLNet_GetError();
	int r = string_new(a);
	string_use( r );
	return r;
}

static int NET_GETERROR (INSTANCE * my, int * params)
{// NET_GetError( error_id );
	int r;
	
	if(params[0]>0)
		return NET_GETERROR_OLD();

	switch(params[0])
	{
	case NET_ERROR_NONE:
		r = string_new("No error");
		break;
	case NET_ERROR_INITIALIZATION:	
		r = string_new("Could not initialize");
		break;
	case NET_ERROR_NOTINITIALIZED:	
		r = string_new("Not initialized");
		break;
	case NET_ERROR_INVALIDADDRESS:	
		r = string_new("Invalid IP address");
		break;
	case NET_ERROR_RESOLVINGHOST:		
		r = string_new("Could not resolve host");
		break;
	case NET_ERROR_ALLOCSOCKETSET:	
		r = string_new("Could not allocate socketset");
		break;
	case NET_ERROR_INVALIDSOCKETSET:	
		r = string_new("Invalid socketset");
		break;
	case NET_ERROR_SOCKETSETINACTIVE:	
		r = string_new("Socketset inactive");
		break;
	case NET_ERROR_ADDINGSOCKET:		
		r = string_new("Could not add socket");
		break;
	case NET_ERROR_INVALIDSOCKET:		
		r = string_new("Invalid socket");
		break;
	case NET_ERROR_DELETINGSOCKET:	
		r = string_new("Could not delete socket");
		break;
	case NET_ERROR_CONNECTING:	
		r = string_new("Could not open connection");
		break;
	case NET_ERROR_INVALIDCONN:		
		r = string_new("Invalid connection");
		break;
	case NET_ERROR_CONNINACTIVE:		
		r = string_new("Connection inactive");
		break;
	case NET_ERROR_LISTENINGONCONN:	
		r = string_new("Could not open listenport");
		break;
	case NET_ERROR_TOOMANYCONNS:		
		r = string_new("Too many connections");
		break;
	case NET_ERROR_TOOFEWCONNS:
		r = string_new("Too few connections");
		break;
	case NET_ERROR_INVALIDTYPE:
		r = string_new("Wrong connection type");
		break;
	case NET_ERROR_ALREADYINIT:
		r = string_new("Already initialized");
		break;
	case NET_ERROR_MESSAGETOOLONG:
		r = string_new("Message too long");
		break;
	case NET_ERROR_MESSAGETOOSHORT:
		r = string_new("Message too short");
		break;
	case NET_ERROR_OUTGOINGSTACKFULL:
		r = string_new("Outgoing Message stack is full");
		break;
	case NET_ERROR_INCOMINGSTACKFULL:
		r = string_new("Incoming Message stack is full");
		break;
	case NET_ERROR_SENDING:
		r = string_new("Could not send message");
		break;
	case NET_ERROR_RECEIVING:
		r = string_new("Could not receive message");
		break;
	case NET_ERROR_DISCONNECTING:
		r = string_new("Could not close connection");
		break;
	case NET_ERROR_DISCONNECTED:
		r = string_new("Connection was closed");
		break;
	case NET_ERROR_TOOMANYLISTENPORTS:
		r = string_new("Too many listenports");
		break;
	case NET_ERROR_INVALIDPORTNUMBER:
		r = string_new("Invalid port number");
		break;
	case NET_ERROR_SIZEMISMATCH:
		r = string_new("Integral size mismatch");
		break;
	case NET_ERROR_RESOLVINGIP:
		r = string_new("Could not resolve IP");
		break;
	case NET_ERROR_NONEXTERNALCONN:
		r = string_new("Non-external connection");
		break;
	case NET_ERROR_NONEWMESSAGE:
		r = string_new("No new message");
		break;
	case NET_ERROR_UNFINISHEDMESSAGE:
		r = string_new("Unfinished message");
		break;
	case NET_ERROR_NULLPOINTER:
		r = string_new("NULL pointer exception\n");
		break;
	case NET_ERROR_NONEXISTINGGRAPH:
		r = string_new("Graph does not exist\n");
		break;
	case NET_ERROR_NONEXISTINGFILE:
		r = string_new("File does not exist\n");
		break;
	case NET_ERROR_CREATINGGRAPH:
		r = string_new("Could not create graph\n");
		break;
	case NET_ERROR_OPENINGFILE:
		r = string_new("Could not open file\n");
		break;
	default:
		r = string_new("Unknown error");
		break;
	}
	string_use( r );

	return r;

}

static int NET_LISTEN (INSTANCE * my, int * params)
{// NET_Listen( port );
	return NET_Listen(0,(Uint16)params[0],NET_CONSOLEREPORTS_NO);
}

static int NET_LISTEN2 (INSTANCE * my, int * params)
{// NET_Listen( port , consolereports on/off );
	return NET_Listen(0,(Uint16)params[0],(Sint8)params[1]);
}

static int NET_IPADDRESS (INSTANCE * my, int * params)
{// NET_IPAddress( conn );
	int r;
	Uint8 s[16];
	Uint8 i;
	Uint16 conn = (Uint16)params[0];
	Uint8* ipp;
	if(conn<0 || conn>=NET.maxconnections)
	{
		if(*NET.consolereports)
			gr_con_printf("[NET] Error: Connection %i invalid\n",conn);
		return NET_ERROR_INVALIDCONN;
	}
	ipp = (Uint8*)&NET.Conn[conn].address.host;

	i = 0;
	for(r=0; r<4; r++)
	{
		itoa(ipp[r],&s[i],10);
		i+=(Uint8)log10(ipp[r])+1;
		s[i] = '.';
		i++;
	}
	s[i-1] = '\0';
	r=string_new(&s[0]);
	string_use(r);
	return r;

}

static int NET_PORT (INSTANCE * my, int * params)
{// NET_Port( conn );
	Uint16 conn = (Uint16)params[0];
	Uint16 p;
	Uint8* bp;
	if(conn<0 || conn>=NET.maxconnections)
	{
		if(*NET.consolereports)
			gr_con_printf("[NET] Error: Connection %i invalid\n",conn);
		return NET_ERROR_INVALIDCONN;
	}
//	if(NET.Status[conn]<=NET_STATUS_INACTIVE)
//	{
//		if(*NET.consolereports)
//			gr_con_printf("[NET] Error: Connection %i not active\n",conn);
//		return NET_ERROR_CONNINACTIVE;
//	}
	bp = (Uint8*)&NET.Conn[conn].address.port;
	p = bp[1]+256*bp[0];
	return SDLNet_Read16(&NET.Conn[conn].address.port);
}

static int NET_HOSTNAME (INSTANCE * my, int * params)
{// NET_Hostname( conn );

	Uint16 conn = (Uint16)params[0];
	Uint8* s;
	int r;

	if(conn<0 || conn>=NET.maxconnections)
	{
		if(*NET.consolereports)
			gr_con_printf("[NET] Error: Connection %i invalid\n",conn);
		r=string_new("");
		string_use(r);
		return r;
	}
//	if(NET.Status[conn]==NET_STATUS_INACTIVE)
//	{
//		if(*NET.consolereports)
//			gr_con_printf("[NET] Error: Connection %i not active\n",conn);
//		r=string_new("");
//		string_use(r);
//		return r;
//	}

	s = (Uint8*)SDLNet_ResolveIP(&NET.Conn[conn].address);
	if( !s )
	{
		if(*NET.consolereports)
			gr_con_printf("[NET] Error: could not resolve ip\n");
		r=string_new("");
		string_use(r);
		return r;
	}

	r=string_new(s);
	string_use(r);
	return r;
}

static int NET_RESOLVE (INSTANCE * my, int * params)
{// NET_ResolveIP( string hostname );
	Uint8* s;
	Uint8* hostname = (Uint8*)string_get(params[0]);
	IPaddress ipa;
	int r;

	if( SDLNet_ResolveHost(&ipa,hostname,0) == -1 )
	{
		if(*NET.consolereports)
			gr_con_printf("[NET] Error: could not resolve host\n");
		r=string_new("");
		string_use(r);
		string_discard(params[0]);
		return r;
	}
	s = (Uint8*)SDLNet_ResolveIP(&ipa);
	if(s==NULL)
	{
		if(*NET.consolereports)
			gr_con_printf("[NET] Error: could not resolve ip\n");
		r=string_new("");
		string_use(r);
		string_discard(params[0]);
		return r;
	}

	r=string_new(s);
	string_use(r);
	string_discard(params[0]);
	return r;
}

static int NET_SEPARATOR (INSTANCE * my, int * params)
{// NET_Separator( conn , separator , separatorlength );
	int r;
	if(params[1]==0||params[2]==0) // delete the separator
	{
		r = NET_Separator((Uint16)params[0],NULL,(Uint8)params[2]);
	}
	else // set the separator
	{
		Uint8* s = (Uint8*)string_get(params[1]);
		r = NET_Separator((Uint16)params[0],s,(Uint8)params[2]);
		string_discard(params[1]);
	}
	return r;
}
/*
static int NET_SEPARATOR2 (INSTANCE * my, int * params)
{// NET_Separator( conn , separator , separatorlength , includeseparator yes/no );
	int r;
	if(params[1]==0) // delete the separator
	{
		r = NET_Separator((Uint16)params[0],NULL,(Uint8)params[2],(Uint8)params[3]);
	}
	else // set the separator
	{
		Uint8* s = (Uint8*)string_get(params[1]);
		r = NET_Separator((Uint16)params[0],s,(Uint8)params[2],(Uint8)params[3]);
		string_discard(params[1]);
	}
	return r;
}
*/
static int NET_GETSEPARATOR (INSTANCE * my, int * params)
{// NET_GetSeparator( conn );
	int r;
	int conn = (Uint16)params[0];
	if(conn<0 || conn>=NET.maxconnections)
	{
		if(*NET.consolereports)
			gr_con_printf("[NET] Error: Connection %i invalid\n",conn);
		r=string_new("");
		string_use(r);
		return r;
	}
	if(NET.Conn[conn].separatorlength>0)
		r = string_new(NET.Conn[conn].separator);
	else
		r = string_new("");
	string_use(r);
	return r;
}

static int NET_GETSEPARATORLENGTH (INSTANCE * my, int * params)
{// NET_GetSeparatorLength( conn );
	if(params[0]<0 || params[0]>=NET.maxconnections)
	{
		if(*NET.consolereports)
			gr_con_printf("[NET] Error: Connection %i invalid\n",params[0]);
		return NET_ERROR_INVALIDCONN;
	}
	return NET.Conn[params[0]].separatorlength;
}

static int NET_IPTOINT (INSTANCE * my, int * params)
{// NET_IPToInt( ip );
	Uint8 ipd[4] = {0,0,0,0};
	Uint8 ipdn;
	Uint8 p;
	int len;
	int i;
	Uint8 tempvalue[2] = {0,0};

	Uint8* ip = (Uint8*)string_get(params[0]);
	if(ip==NULL)
		return 0;


	len = strlen(ip);
	p=1;
	ipdn = 3;
	for(i=len-1; i>=0; i--)
	{
		if(ip[i]=='.')
		{
			p=1;
			ipdn--;
			i--;
		}
		tempvalue[0] = ip[i];
		ipd[ipdn]+=atoi((const Uint8*)&tempvalue[0])*p;
		p*=10;
	}

	return ipd[0]*256*256*256+ipd[1]*256*256+ipd[2]*256+ipd[3];
}

static int NET_INTTOIP (INSTANCE * my, int * params)
{// NET_IntToIP( int_ip );

	int r;
	int i;
	Uint8 ip[16];
	Uint8 ipd[4];
	Uint32 ipi = params[0];
	if(ipi<0)
	{
		ipi+=2147483648;//pow(2,31);
		ipd[0] = 128+ipi/256/256/256;
	}
	else
		ipd[0] = ipi/256/256/256;

	ipd[1] = ipi/256/256 - ipd[0]*256*256*256;
	ipd[2] = ipi/256 - ipd[1]*256*256;
	ipd[3] = ipi - ipd[2]*256;

	i = 0;
	for(r=0; r<4; r++)
	{
		itoa(ipd[r],&ip[i],10);
		i+=(Uint8)log10(ipd[r])+1;
		ip[i] = '.';
		i++;
	}
	ip[i-1] = '\0';
	r=string_new(&ip[0]);
	string_use(r);
	return r;
}

static int NET_STAT_BUFFER (INSTANCE * my, int * params)
{// NET_Stat_Buffer( conn );

	int r;
	Uint16 conn = (Uint16)params[0];
	if(conn<0 || conn>=NET.maxconnections)
	{
		if(*NET.consolereports)
			gr_con_printf("[NET] Error: Connection %i invalid\n",conn);
		r=string_new("");
		string_use(r);
		return r;
	}
	if(NET.Status[conn]==NET_STATUS_INACTIVE)
	{
		if(*NET.consolereports)
			gr_con_printf("[NET] Error: Connection %i not active\n",conn);
		r=string_new("");
		string_use(r);
		return r;
	}

	r = string_new(NET.Conn[conn].incomingmsg);
	string_use(r);
	return r;
}

static int NET_CONSOLEMESSAGE (INSTANCE * my, int * params)
{// NET_ConsoleMessage( conn, message );
	Uint8* s;
	if(params[0]<0 || params[0]>=NET.maxconnections)
		if(!NET.Conn[params[0]].consolereports)
			return NET_ERROR_NONE;
	else
		if(*NET.consolereports)
			return NET_ERROR_NONE;
	s = (Uint8*)string_get(params[1]);
	gr_con_printf("[NET] %s\n",s);
	string_discard(params[1]);
	return NET_ERROR_NONE;
}

/********************************************************************\
*                                                                    *
* Export Fenix Functions                                     Sandman *
*                                                                    *
\********************************************************************/

FENIX_MainDLL RegisterFunctions (COMMON_PARAMS)
{
    FENIX_DLLImport

	// Information
	FENIX_export ("NET_ABOUT",				"",		TYPE_DWORD,		NET_ABOUT				) ;
	FENIX_export ("NET_VERSION",			"",		TYPE_STRING,	NET_VERSION				) ;
	FENIX_export ("NET_INTVERSION",			"",		TYPE_DWORD,		NET_INTVERSION			) ;

	// Initialize
	FENIX_export ("NET_DLLINIT",			"IIIPPPPPP",TYPE_DWORD,	NET_INIT				) ;
	FENIX_export ("NET_DUMINIT",			"PPPPPPPP",	TYPE_DWORD,	NET_DUMINIT				) ;
	FENIX_export ("NET_DLLQUIT",			"",		TYPE_DWORD,		NET_QUIT				) ;

	// Loop
	FENIX_export ("NET_LOOP",				"",		TYPE_DWORD,		NET_LOOP				) ;

	// Connection
	FENIX_export ("NET_CONNECT",			"SI",	TYPE_DWORD,		NET_CONNECT				) ;
	FENIX_export ("NET_CONNECT",			"SII",	TYPE_DWORD,		NET_CONNECT2			) ;
	FENIX_export ("NET_OPEN",				"SI",	TYPE_DWORD,		NET_CONNECT				) ;
	FENIX_export ("NET_OPEN",				"SII",	TYPE_DWORD,		NET_CONNECT2			) ;
	FENIX_export ("NET_LISTEN",				"I",	TYPE_DWORD,		NET_LISTEN				) ;
	FENIX_export ("NET_LISTEN",				"II",	TYPE_DWORD,		NET_LISTEN2				) ;
	FENIX_export ("NET_DISCONNECT",			"I",	TYPE_DWORD,		NET_DISCONNECT			) ;
	FENIX_export ("NET_CLOSE",				"I",	TYPE_DWORD,		NET_DISCONNECT			) ;
	FENIX_export ("NET_DISCONNECTALL",		"",		TYPE_DWORD,		NET_DISCONNECTALL		) ;
	FENIX_export ("NET_CLOSEALL",			"",		TYPE_DWORD,		NET_DISCONNECTALL		) ;

	// Connection information
	FENIX_export ("NET_PORT",				"I",	TYPE_DWORD,		NET_PORT				) ;
	FENIX_export ("NET_IPADDRESS",			"I",	TYPE_STRING,	NET_IPADDRESS			) ;
	FENIX_export ("NET_HOSTNAME",			"I",	TYPE_STRING,	NET_HOSTNAME			) ;

	// Message (send/recv)
	FENIX_export ("NET_SEND",				"IS",	TYPE_DWORD,		NET_SEND				) ;
	FENIX_export ("NET_MESSAGE",			"IS",	TYPE_DWORD,		NET_SEND				) ;
	FENIX_export ("NET_SEND",				"ISI",	TYPE_DWORD,		NET_SEND2				) ;
	FENIX_export ("NET_MESSAGE",			"ISI",	TYPE_DWORD,		NET_SEND2				) ;
	FENIX_export ("NET_SENDRN",				"IS",	TYPE_DWORD,		NET_SENDRN				) ;
	FENIX_export ("NET_MESSAGERN",			"IS",	TYPE_DWORD,		NET_SENDRN				) ;
	FENIX_export ("NET_RECV",				"I",	TYPE_STRING,	NET_RECV				) ;
	FENIX_export ("NET_GETMESSAGE",			"I",	TYPE_STRING,	NET_RECV				) ;
//	FENIX_export ("NET_MESSAGE",			"I",	TYPE_STRING,	NET_RECV				) ;
	FENIX_export ("NET_RECV",				"II",	TYPE_STRING,	NET_RECV2				) ;
	FENIX_export ("NET_GETMESSAGE",			"II",	TYPE_STRING,	NET_RECV2				) ;
//	FENIX_export ("NET_MESSAGE",			"II",	TYPE_STRING,	NET_RECV2				) ;

	// Variable (send/recv)
	FENIX_export ("NET_SENDVAR",			"IPI",	TYPE_DWORD,		NET_SENDVAR				) ;
	FENIX_export ("NET_SENDVAR",			"IPII",	TYPE_DWORD,		NET_SENDVAR2			) ;
	FENIX_export ("NET_RECVVAR",			"IPI",	TYPE_DWORD,		NET_RECVVAR				) ;
	FENIX_export ("NET_RECVVAR",			"IPII",	TYPE_DWORD,		NET_RECVVAR2			) ;

	// Separator
	FENIX_export ("NET_SEPARATOR",			"ISI",	TYPE_DWORD,		NET_SEPARATOR			) ;
	FENIX_export ("NET_GETSEPARATOR",		"I",	TYPE_STRING,	NET_GETSEPARATOR		) ;
	FENIX_export ("NET_SEPARATOR",			"I",	TYPE_STRING,	NET_GETSEPARATOR		) ;
	FENIX_export ("NET_GETSEPARATORLENGTH",	"I",	TYPE_DWORD,		NET_GETSEPARATORLENGTH	) ;

	// Other
	FENIX_export ("NET_RESOLVE",			"S",	TYPE_STRING,	NET_RESOLVE				) ;
	FENIX_export ("NET_GETERROR",			"I",	TYPE_STRING,	NET_GETERROR			) ;
	FENIX_export ("NET_IPTOINT",			"S",	TYPE_DWORD,		NET_IPTOINT				) ;
	FENIX_export ("NET_INTTOIP",			"I",	TYPE_STRING,	NET_INTTOIP				) ;

	// Status
	FENIX_export ("NET_STAT_BUFFER",		"I",	TYPE_STRING,	NET_STAT_BUFFER			) ;

	// Other
	FENIX_export ("NET_CONSOLEMESSAGE",		"IS",	TYPE_DWORD,		NET_CONSOLEMESSAGE		) ;
	FENIX_export ("NET_DLLSEND",			"IPII",	TYPE_DWORD,		NET_DLLSEND				) ;
	FENIX_export ("NET_DLLRECVVAR",			"IPII",	TYPE_DWORD,		NET_DLLRECVVAR			) ;
}

/*

23-04-07:
	Fixed a "debug assertion failed" error, when sending something, without having specified
		a separator. This was due to the freeing of a Fenix string in NET_Send(), because
		of the lack of a check. 

26-03-07:
	Renamed all functions related to "seperator" to "separator".

12-02-07:
	Fixed bug in NET_ConsoleMessages(), causing the string system to fail.
	Moved the parameter includeseparator from NET_Separator() to NET_Recv() and NET_RecvVar().
	Removed the variable separatorincluded from the connection struct.
	Added errormessages for the new errorcodes in NET_GetError().

10-02-07:
	Added NET_DLLSend().
	Added NET_DLLRecvVar().
	Added NET_SeparatorLength();

04-02-07:
	Added NET_ConsoleMessage().
	Changed the about text in NET_About().
	Moved the checks in NET_Send() to the functions using it.
	Moved the checks in NET_Recv() and NET_RecvVar() to the functions using them.

02-02-07:
	Added the check if there was a separator specified to NET_Send(), if one was wanted at the end
	Added the freeing of old separator data if a new one is specified.
	Added a separator is only removed when there is none.
	Added if params[2] of NET_SEPARATOR is 0, then deleting of separator is also carried out.

26-01-07:
	Added the parameter includeseparator to NET_Send(). When it is not 0, the connection's
		separator will be added at the end of the to be sent data. When it is 0, it won't.

25-01-07:
	Changed NET_Recv() for more efficiency.
	Made messages unlimited. because in v1.5x this was more easy. NET_RECV() still has a limit
		of NET_MAX_MSG_LEN, but this is not a low number, so all should be well. If it's required,
		it is still possible to make that one unlimited as well.
	Removed the maxlen parameter of NET_Recv(), it didn't have much use anymore.

24-01-07:
	Added NET_ERROR_NULLPOINTER.
	Changed NET_RecvVar() for more efficiecy.

23-01-07:
	Added NET_IntToIP() and NET_IPToInt(),which were previously made in Fenix
	Commented out the timeoutcounter in the connection datatype (it wasn't used anymore)
	Fixed if other peer closes, this side does not notice until it tries to send a message.
	    (this was done by changing r<0 to r<=0 in NET_GetIncomingMsg())

16-01-07: v1.52:
	Added comments and tidied up.
	Added and changed some console messages.

15-01-07:
	Added comments, tidied up the code a little.
	Added the possibility of removing separators.
	Added NET_DisconnectAll() and its synonym NET_CloseAll().
	Added NET_IntVersion().

08-01-07: v1.51:
	Released!

07-01-07: v1.50epsilon:
	Perfect the shit (well sort of). And yes, I know the Greek alphabet.

06-01-07: v1.50delta:
	It seems everything is in working order, after some hardwork. The commands have
	changed slightly, I'm afraid backwards compatibility is not an option. Therefore
	I'll try to list the changes as soon as possible and update the documentation.

05-01-07: v1.50gamma:
	Almost done now, I'd say. Most of the code is rewritten to a new system and a few
	functions have been added. I'll try to list all the changes/additions when I'm done.
	Half of the documentation is adjusted to the new code.

04-01-07: v1.50beta:
	And here it is. I've gone through all of the code now, editing stuff here and there,
	where I knew what had to be done. I Started contemplating more functionality and more
	effeciency.

03-01-07: v1.50alpha:
	Started the recoding of Network.DLL from v1.14. The code of newer versions was lost in a
	format. Yes, it was a sad day indeed. Not to worry though, I got the basis systems up
	and running. More will follow.

03-07-06: v1.14:
	Fixed bug causing to send only to first 255 connections with NET_MessageRN().
	Changed Fenix NET_MessageRN()'s name to NET_MessageRN2().
	Added DLLside NET_MessageRN().
	Renamed NET_Message3() to NET_Message2().
	Added Global Vars NET.ActiveConnections and NET.ActiveListenConnections.
	Changed consolereports and returnnetdllcommands to pointers.
	Added NET_Hostname().
	Added NET_Port().
	Added NET_Resolve().
	Added a check in NET_Disconnect() and NET_Accept() if (dis)connecting/disconnected hostname
	was found.
	Added NET_DUMInit().
	Added various global variables for the DU Meter.
	Fixed some messaging functions not returning errorcodes.
	From now on the version will be 1.xx.

29-06-06: v0.13
	Changed NET_Listen() to be accessible from Fenix.
	Added parameter <numberoflistenports> to NET_Init().
	Added parameter <consolereports> to various functions.
	Added errorcodes.
	(DLL) Added parameter <returnnetdllmessages> to NET_DLLInit();
	Fixed some messaging functions not returning errorcodes.
	Anything else I did, like fixing the messagelosses.

11-06-06: v0.12
	Fixed flooding when other peer disconnects.
	Fixed previous messages traces in current message.
	Added errorcodes.
	Added NET_Message2() and NET_Message3().
	Fixed NET_Quit() now also kills the NET_Run().
	Added NET_Run() as part of the NET_Init().
	Changed name from NET.DLL to Network.DLL to avoid confusion.

10-06-06: v0.11
	Fixed some bugs and recoded parts of the core.

08-06-06: v0.10
	Released
	(DLL) Changed NET_Init() to NET_DLLInet().

06-06-06: v0.09
	Added some defines.
	Added NET_Init() return an error if the maximum number of connections is wrong.

v0.08:
	Added defines, made the returns return them and made more error checks.

v0.07:
	Added Global Status Identifier.

v0.05:
	removed bug: couldn't send more than 3 messages using NET_Message, though NET_Send worked
	fine. Fixed it by letting NET_Message return 0.

30-04-06: v0.04
	Added some defines.

v0.01-v0.04:
	Created main functions and so on.
*/